==> oer-0.0.1-20190731.023331-5-sources.jar: com/xdja/pki/itsca/oer/app/data/CertDataBuilder.java <==
package com.xdja.pki.itsca.oer.app.data;

import com.xdja.pki.itsca.oer.app.bean.PKRecipientInfoType;
import com.xdja.pki.itsca.oer.asn1.*;
import com.xdja.pki.itsca.oer.asn1.base.Enumerated;
import com.xdja.pki.itsca.oer.asn1.base.OctetString;
import com.xdja.pki.itsca.oer.cert.TbsCertBuilder;
import com.xdja.pki.itsca.oer.utils.*;
import org.bouncycastle.util.BigIntegers;
import org.bouncycastle.util.encoders.Hex;

import java.security.PublicKey;
import java.security.interfaces.ECPublicKey;

/**
 * @ClassName CertDataBuilder
 * @Description TODO
 * @Date 2019/7/12 14:00
 * @Author FengZhen
 */
public class CertDataBuilder {
    // TODO: 2019/7/12 私钥是否要对TbsCert进行签名
    CertRequest buildCertRequest(PublicKey certPublicKey, PublicKey temporaryPublic, String itsId, String subjectName, Enumerated.Value subjectTypeValue) throws Exception {
        TbsCertBuilder tbsCertBuilder = new TbsCertBuilder();
        SequenceOfItsAidList sequenceOfItsAidList = new SequenceOfItsAidList();

        sequenceOfItsAidList.addItsAid(ItsAid.TRAFFIC_LIGHT_APPLICATION);
        sequenceOfItsAidList.addItsAid(ItsAid.NON_REGISTRATION_CERTIFICATE_REQUEST);
        sequenceOfItsAidList.addItsAid(ItsAid.REGISTRATION_CERTIFICATE_RESPONSE);
        tbsCertBuilder.setItsAidList(sequenceOfItsAidList);

        GeographicRegion geographicRegion = new GeographicRegion();
        SequenceOfRectangularRegion sequenceOfRectangularRegion = new SequenceOfRectangularRegion();
        RectangularRegion rectangularRegion = new RectangularRegion();
        TwoDLocation twoDLocation = new TwoDLocation();
        twoDLocation.setLongitude(new Longitude(57834));
        twoDLocation.setLatitude(new Latitude(4897202));
        rectangularRegion.setSouthEast(twoDLocation);
        twoDLocation.setLongitude(new Longitude(547834));
        twoDLocation.setLatitude(new Latitude(985434));
        rectangularRegion.setNorthWest(twoDLocation);
        sequenceOfRectangularRegion.addRectangularRegion(rectangularRegion);
        geographicRegion.setRectangularRegion(sequenceOfRectangularRegion);
        tbsCertBuilder.setGeographicRegion(geographicRegion);
        TbsCert tbsCert = tbsCertBuilder.build(certPublicKey, certPublicKey, subjectName, subjectTypeValue);

        CertRequest certRequest = new CertRequest();
        OctetString id = new OctetString();
        id.setString(itsId.getBytes());
        byte[] decode = Hex.decode("33303832303331313330383230314639");
        OctetString octetString = new OctetString();
        octetString.setString(decode);
        certRequest.setItsId(decode);

        long nowTime = TimeUtils.getNowTime();
        Time32 currentTime = new Time32(nowTime);
        certRequest.setCurrentTime(currentTime);

        certRequest.setTbsCertData(tbsCert);


        PublicEncryptionKey publicEncryptionKey = new PublicEncryptionKey();
        publicEncryptionKey.setEccCurve(new EccCurve(EccCurve.SGD_SM2));
        publicEncryptionKey.setSupportedSymmAlg(new SymmetricAlgorithm(SymmetricAlgorithm.SGD_SM4_ECB));

        Uncompressed uncompressed = new Uncompressed();
        ECPublicKey ecPublicKey = (ECPublicKey) temporaryPublic;
        byte[] x = BigIntegers.asUnsignedByteArray(ecPublicKey.getW().getAffineX());
        byte[] y = BigIntegers.asUnsignedByteArray(ecPublicKey.getW().getAffineY());
        uncompressed.setX(x);
        uncompressed.setY(y);


        EccPoint eccPoint = new EccPoint();
        eccPoint.setUncompressed(uncompressed);
        publicEncryptionKey.setPublicKey(eccPoint);

        certRequest.setEncryptionKey(publicEncryptionKey);
        ByteArrayUtils.printHexBinary(null, "certRequest", certRequest.getEncode());
        return certRequest;
    }

    SecuredMessage buildSecuredMessage(CertRequest certRequest, boolean isSignedDataRecip, byte[] key, byte[] data) throws Exception {
        SecuredMessage securedMessage = new SecuredMessage();
        PKRecipientInfo pkRecipientInfo = new PKRecipientInfo();

        HashAlgorithm hashAlg = new HashAlgorithm(HashAlgorithm.SGD_SM3);
        pkRecipientInfo.setHashAlg(hashAlg);
        //中的CertRequest数据的OER编码做HashID8计算并对
        byte[] certReqHash = BCUtils.sm3DigestByBC(certRequest.getEncode());
        byte[] certHashId8 = new byte[8];
        System.arraycopy(certReqHash, certReqHash.length - certHashId8.length, certHashId8, 0, certHashId8.length);
        ByteArrayUtils.printHexBinary(null, "hashID data ", certHashId8);
        HashedId8 recipientId = new HashedId8(certHashId8);
        //EnrollmentCertRespData->payload->encData->recipients->pkRecipientInfo->recipientId赋值，
        pkRecipientInfo.setRecipientId(recipientId);
        //构建kek
        EccPoint publicKey = certRequest.getEncryptionKey().getPublicKey();
        EciesEncryptedKey kek = KekBuilder.build(publicKey, key);
        pkRecipientInfo.setKek(kek);

        EncryptedData encData = new EncryptedData();
        SequenceOfRecipientInfo recipients = new SequenceOfRecipientInfo();
        RecipientInfo recipientInfo;
        if (isSignedDataRecip) {
            recipientInfo = new RecipientInfo(pkRecipientInfo, PKRecipientInfoType.SIGNEDDATA_RECIPINFO);
        } else {
            recipientInfo = new RecipientInfo(pkRecipientInfo, PKRecipientInfoType.CERT_RECIPINFO);
        }

        recipients.addRecipientInfo(recipientInfo);
        encData.setRecipients(recipients);
        //使用传入的key进行ECB加密数据
        byte[] bytes = BCUtils.symmetricECBEncryptByBC(true, BCUtils.EncryptTypeByBC.SM4_ECB_PKCS7Padding, key, data);
        ByteArrayUtils.printHexBinary(null, "BCUtils bytes", bytes);
        CipherText cipherData = new CipherText();
        cipherData.setString(bytes);
        SymmetricCipherText cipherText = new SymmetricCipherText(cipherData);
        //写入密文
        encData.setCipherText(cipherText);
        Payload payload = new Payload(encData);
        securedMessage.setPayload(payload);
        return securedMessage;
    }

    SecuredMessage buildSecuredMessage(Certificate pcaCert, EccPoint publicKey, boolean isSignedDataRecip, byte[] key, byte[] data) throws Exception {
        SecuredMessage securedMessage = new SecuredMessage();
        PKRecipientInfo pkRecipientInfo = new PKRecipientInfo();
        HashAlgorithm hashAlg = new HashAlgorithm(HashAlgorithm.SGD_SM3);
        pkRecipientInfo.setHashAlg(hashAlg);

        byte[] certReqHash = BCUtils.sm3DigestByBC(pcaCert.getEncode());
        byte[] certHashId8 = new byte[8];
        System.arraycopy(certReqHash, certReqHash.length - certHashId8.length, certHashId8, 0, certHashId8.length);
        ByteArrayUtils.printHexBinary(null, "hashID data ", certHashId8);
        HashedId8 recipientId = new HashedId8(certHashId8);
        //EnrollmentCertRespData->payload->encData->recipients->pkRecipientInfo->recipientId赋值，
        pkRecipientInfo.setRecipientId(recipientId);
        //构建kek
        EciesEncryptedKey kek = KekBuilder.build(publicKey, key);
        pkRecipientInfo.setKek(kek);
        EncryptedData encData = new EncryptedData();
        SequenceOfRecipientInfo recipients = new SequenceOfRecipientInfo();
        RecipientInfo recipientInfo;
        if (isSignedDataRecip) {
            recipientInfo = new RecipientInfo(pkRecipientInfo, PKRecipientInfoType.SIGNEDDATA_RECIPINFO);
        } else {
            recipientInfo = new RecipientInfo(pkRecipientInfo, PKRecipientInfoType.CERT_RECIPINFO);
        }
        recipients.addRecipientInfo(recipientInfo);
        encData.setRecipients(recipients);
        //使用传入的key进行ECB加密数据
        byte[] bytes = BCUtils.symmetricECBEncryptByBC(true, BCUtils.EncryptTypeByBC.SM4_ECB_PKCS7Padding, key, data);
        ByteArrayUtils.printHexBinary(null, "BCUtils bytes", bytes);
        CipherText cipherData = new CipherText();
        cipherData.setString(bytes);
        SymmetricCipherText cipherText = new SymmetricCipherText(cipherData);
        //写入密文
        encData.setCipherText(cipherText);
        Payload payload = new Payload(encData);
        securedMessage.setPayload(payload);
        return securedMessage;
    }

}

==> oer-0.0.2-20190823.092113-22-sources.jar: com/xdja/pki/itsca/oer/app/data/CertDataBuilder.java <==
package com.xdja.pki.itsca.oer.app.data;

import com.xdja.pki.gmssl.crypto.sdf.SdfCryptoType;
import com.xdja.pki.gmssl.crypto.sdf.SdfSymmetricKeyParameters;
import com.xdja.pki.gmssl.crypto.utils.GMSSLSM3DigestUtils;
import com.xdja.pki.gmssl.crypto.utils.GMSSLSymmetricEncryptUtils;
import com.xdja.pki.gmssl.sdf.bean.SdfAlgIdSymmetric;
import com.xdja.pki.itsca.oer.app.bean.PKRecipientInfoType;
import com.xdja.pki.itsca.oer.asn1.*;
import com.xdja.pki.itsca.oer.asn1.base.Enumerated;
import com.xdja.pki.itsca.oer.asn1.base.OctetString;
import com.xdja.pki.itsca.oer.cert.TbsCertBuilder;
import com.xdja.pki.itsca.oer.utils.*;
import org.bouncycastle.util.BigIntegers;
import org.bouncycastle.util.encoders.Hex;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.security.PublicKey;
import java.security.interfaces.ECPublicKey;

/**
 * @ClassName CertDataBuilder
 */
public class CertDataBuilder {
    private Logger logger = LoggerFactory.getLogger(this.getClass());


    CertRequest buildCertRequest(PublicKey certPublicKey, PublicKey temporaryPublic, String itsId, String subjectName, Enumerated.Value subjectTypeValue) throws Exception {
        TbsCertBuilder tbsCertBuilder = new TbsCertBuilder();
        SequenceOfItsAidList sequenceOfItsAidList = new SequenceOfItsAidList();

        sequenceOfItsAidList.addItsAid(ItsAid.TRAFFIC_LIGHT_APPLICATION);
        sequenceOfItsAidList.addItsAid(ItsAid.NON_REGISTRATION_CERTIFICATE_REQUEST);
        sequenceOfItsAidList.addItsAid(ItsAid.REGISTRATION_CERTIFICATE_RESPONSE);
        tbsCertBuilder.setItsAidList(sequenceOfItsAidList);

        GeographicRegion geographicRegion = new GeographicRegion();
        SequenceOfRectangularRegion sequenceOfRectangularRegion = new SequenceOfRectangularRegion();
        RectangularRegion rectangularRegion = new RectangularRegion();
        TwoDLocation twoDLocation = new TwoDLocation();
        twoDLocation.setLongitude(new Longitude(57834));
        twoDLocation.setLatitude(new Latitude(4897202));
        rectangularRegion.setSouthEast(twoDLocation);
        twoDLocation.setLongitude(new Longitude(547834));
        twoDLocation.setLatitude(new Latitude(985434));
        rectangularRegion.setNorthWest(twoDLocation);
        sequenceOfRectangularRegion.addRectangularRegion(rectangularRegion);
        geographicRegion.setRectangularRegion(sequenceOfRectangularRegion);
        tbsCertBuilder.setGeographicRegion(geographicRegion);
        TbsCert tbsCert = tbsCertBuilder.build(certPublicKey, certPublicKey, subjectName, subjectTypeValue);

        CertRequest certRequest = new CertRequest();
        OctetString id = new OctetString();
        id.setString(itsId.getBytes());
        byte[] decode = Hex.decode("33303832303331313330383230314639");
        OctetString octetString = new OctetString();
        octetString.setString(decode);
        certRequest.setItsId(decode);

        long nowTime = TimeUtils.getNowTime();
        Time32 currentTime = new Time32(nowTime);
        certRequest.setCurrentTime(currentTime);

        certRequest.setTbsCertData(tbsCert);


        PublicEncryptionKey publicEncryptionKey = new PublicEncryptionKey();
        publicEncryptionKey.setEccCurve(new EccCurve(EccCurve.SGD_SM2));
        publicEncryptionKey.setSupportedSymmAlg(new SymmetricAlgorithm(SymmetricAlgorithm.SGD_SM4_ECB));

        Uncompressed uncompressed = new Uncompressed();
        ECPublicKey ecPublicKey = (ECPublicKey) temporaryPublic;
        byte[] x = BigIntegers.asUnsignedByteArray(ecPublicKey.getW().getAffineX());
        byte[] y = BigIntegers.asUnsignedByteArray(ecPublicKey.getW().getAffineY());
        uncompressed.setX(x);
        uncompressed.setY(y);


        EccPoint eccPoint = new EccPoint();
        eccPoint.setUncompressed(uncompressed);
        publicEncryptionKey.setPublicKey(eccPoint);

        certRequest.setEncryptionKey(publicEncryptionKey);
        ByteArrayUtils.printHexBinary(null, "certRequest", certRequest.getEncode());
        return certRequest;
    }

    SecuredMessage buildSecuredMessage(CertRequest certRequest, boolean isSignedDataRecip, byte[] key, byte[] data) throws Exception {
        SecuredMessage securedMessage = new SecuredMessage();
        PKRecipientInfo pkRecipientInfo = new PKRecipientInfo();

        HashAlgorithm hashAlg = new HashAlgorithm(HashAlgorithm.SGD_SM3);
        pkRecipientInfo.setHashAlg(hashAlg);
        //中的CertRequest数据的OER编码做HashID8计算并对
        //   byte[] certReqHash = BCUtils.sm3DigestByBC(certRequest.getEncode());
        byte[] certReqHash = GMSSLSM3DigestUtils.digestByBC(certRequest.getEncode());


        byte[] certHashId8 = new byte[8];
        System.arraycopy(certReqHash, certReqHash.length - certHashId8.length, certHashId8, 0, certHashId8.length);
        ByteArrayUtils.printHexBinary(null, "hashID data ", certHashId8);
        HashedId8 recipientId = new HashedId8(certHashId8);
        //EnrollmentCertRespData->payload->encData->recipients->pkRecipientInfo->recipientId赋值，
        pkRecipientInfo.setRecipientId(recipientId);
        //构建kek
        EccPoint publicKey = certRequest.getEncryptionKey().getPublicKey();
        EciesEncryptedKey kek = KekBuilder.build(publicKey, key);
        pkRecipientInfo.setKek(kek);

        EncryptedData encData = new EncryptedData();
        SequenceOfRecipientInfo recipients = new SequenceOfRecipientInfo();
        RecipientInfo recipientInfo;
        if (isSignedDataRecip) {
            recipientInfo = new RecipientInfo(pkRecipientInfo, PKRecipientInfoType.SIGNEDDATA_RECIPINFO);
        } else {
            recipientInfo = new RecipientInfo(pkRecipientInfo, PKRecipientInfoType.CERT_RECIPINFO);
        }

        recipients.addRecipientInfo(recipientInfo);
        encData.setRecipients(recipients);
        //使用传入的key进行ECB加密数据
        //byte[] bytes = BCUtils.symmetricECBEncryptByBC(true, BCUtils.EncryptTypeByBC.SM4_ECB_PKCS7Padding, key, data);
        byte[] bytes = GMSSLSymmetricEncryptUtils.symmetricECBEncryptBySdf(true, SdfCryptoType.YUNHSM,
                SdfSymmetricKeyParameters.PaddingType.PKCS7Padding, key, SdfAlgIdSymmetric.SGD_SM4_ECB, data);
        ByteArrayUtils.printHexBinary(null, "BCUtils bytes", bytes);
        CipherText cipherData = new CipherText();
        cipherData.setString(bytes);
        SymmetricCipherText cipherText = new SymmetricCipherText(cipherData);
        //写入密文
        encData.setCipherText(cipherText);
        Payload payload = new Payload(encData);
        securedMessage.setPayload(payload);
        return securedMessage;
    }

    SecuredMessage buildSecuredMessage(Certificate pcaCert, EccPoint publicKey, boolean isSignedDataRecip, byte[] key, byte[] data) throws Exception {
        SecuredMessage securedMessage = new SecuredMessage();
        PKRecipientInfo pkRecipientInfo = new PKRecipientInfo();
        HashAlgorithm hashAlg = new HashAlgorithm(HashAlgorithm.SGD_SM3);
        pkRecipientInfo.setHashAlg(hashAlg);
//        byte[] certReqHash = BCUtils.sm3DigestByBC(pcaCert.getEncode());
        byte[] certReqHash = GMSSLSM3DigestUtils.digestByBC(pcaCert.getEncode());
        byte[] certHashId8 = new byte[8];
        System.arraycopy(certReqHash, certReqHash.length - certHashId8.length, certHashId8, 0, certHashId8.length);
        ByteArrayUtils.printHexBinary(null, "hashID data ", certHashId8);
        HashedId8 recipientId = new HashedId8(certHashId8);
        //EnrollmentCertRespData->payload->encData->recipients->pkRecipientInfo->recipientId赋值，
        pkRecipientInfo.setRecipientId(recipientId);
        //构建kek
        EciesEncryptedKey kek = KekBuilder.build(publicKey, key);
        pkRecipientInfo.setKek(kek);
        EncryptedData encData = new EncryptedData();
        SequenceOfRecipientInfo recipients = new SequenceOfRecipientInfo();
        RecipientInfo recipientInfo;
        if (isSignedDataRecip) {
            recipientInfo = new RecipientInfo(pkRecipientInfo, PKRecipientInfoType.SIGNEDDATA_RECIPINFO);
        } else {
            recipientInfo = new RecipientInfo(pkRecipientInfo, PKRecipientInfoType.CERT_RECIPINFO);
        }
        recipients.addRecipientInfo(recipientInfo);
        encData.setRecipients(recipients);
        //使用传入的key进行ECB加密数据
        //byte[] bytes = BCUtils.symmetricECBEncryptByBC(true, BCUtils.EncryptTypeByBC.SM4_ECB_PKCS7Padding, key, data);
        byte[] bytes = GMSSLSymmetricEncryptUtils.symmetricECBEncryptBySdf(true, SdfCryptoType.YUNHSM,
                SdfSymmetricKeyParameters.PaddingType.PKCS7Padding, key, SdfAlgIdSymmetric.SGD_SM4_ECB, data);
        ByteArrayUtils.printHexBinary(null, "BCUtils bytes", bytes);
        CipherText cipherData = new CipherText();
        cipherData.setString(bytes);
        SymmetricCipherText cipherText = new SymmetricCipherText(cipherData);
        //写入密文
        encData.setCipherText(cipherText);
        Payload payload = new Payload(encData);
        securedMessage.setPayload(payload);
        return securedMessage;
    }

}
